"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.postUpdate = exports.updateToV10 = exports.updateToV9 = exports.updateToV7 = void 0;
const schematics_1 = require("@angular/cdk/schematics");
const class_names_1 = require("./data/migrations/class-names");
const upgrade_data_1 = require("./upgrade-data");
const calendar_input_rule_1 = require("./upgrade-rules/checks/calendar-input-rule");
const carousel_like_template_rule_1 = require("./upgrade-rules/checks/carousel-like-template-rule");
const date_fns_compatible_rule_1 = require("./upgrade-rules/checks/date-fns-compatible-rule");
const dropdown_class_rule_1 = require("./upgrade-rules/checks/dropdown-class-rule");
const dropdown_template_rule_1 = require("./upgrade-rules/checks/dropdown-template-rule");
const form_template_rule_1 = require("./upgrade-rules/checks/form-template-rule");
const global_config_rule_1 = require("./upgrade-rules/checks/global-config-rule");
const grid_template_rule_1 = require("./upgrade-rules/checks/grid-template-rule");
const icon_template_rule_1 = require("./upgrade-rules/checks/icon-template-rule");
const secondary_entry_points_rule_1 = require("./upgrade-rules/checks/secondary-entry-points-rule");
const table_template_rule_1 = require("./upgrade-rules/checks/table-template-rule");
const tooltip_like_template_rule_1 = require("./upgrade-rules/checks/tooltip-like-template-rule");
const migrations = [
    tooltip_like_template_rule_1.TooltipLikeTemplateRule,
    dropdown_template_rule_1.DropdownTemplateRule,
    dropdown_class_rule_1.DropdownClassRule,
    icon_template_rule_1.IconTemplateRule,
    calendar_input_rule_1.CalendarTemplateRule,
    carousel_like_template_rule_1.CarouselTemplateRule,
    global_config_rule_1.GlobalConfigRule,
    date_fns_compatible_rule_1.DateFnsCompatibleRule,
    form_template_rule_1.FormTemplateRule,
    grid_template_rule_1.GridTemplateRule,
    table_template_rule_1.TableTemplateRule,
    secondary_entry_points_rule_1.SecondaryEntryPointsRule,
    class_names_1.ClassNamesMigration
];
/** Entry point for the migration schematics with target of NG-ZORRO v7 */
function updateToV7() {
    return schematics_1.createMigrationSchematicRule(schematics_1.TargetVersion.V7, migrations, upgrade_data_1.ruleUpgradeData, postUpdate);
}
exports.updateToV7 = updateToV7;
/** Entry point for the migration schematics with target of NG-ZORRO v9 */
function updateToV9() {
    return schematics_1.createMigrationSchematicRule(schematics_1.TargetVersion.V9, migrations, upgrade_data_1.ruleUpgradeData, postUpdate);
}
exports.updateToV9 = updateToV9;
/** Entry point for the migration schematics with target of NG-ZORRO v10 */
function updateToV10() {
    return schematics_1.createMigrationSchematicRule(schematics_1.TargetVersion.V10, migrations, upgrade_data_1.ruleUpgradeData, postUpdate);
}
exports.updateToV10 = updateToV10;
/** Post-update schematic to be called when update is finished. */
function postUpdate(context, targetVersion, hasFailures) {
    context.logger.info('');
    context.logger.info(`  ✓  Updated NG-ZORRO to ${targetVersion}`);
    context.logger.info('');
    if (hasFailures) {
        context.logger.warn('  ⚠  Some issues were detected but could not be fixed automatically. Please check the ' +
            'output above and fix these issues manually.');
    }
}
exports.postUpdate = postUpdate;
//# sourceMappingURL=index.js.map